@@ -24,9 +24,8 @@ Known hash commands :
   - #bpp              # bytes per panel (2 by default)
 """
 
-from decimal import Decimal
 from numbers import Number
-from typing import Any, Iterable, List, Optional, Tuple, Union
+from typing import Any, List, Optional, Tuple
 
 from parsimonious import Grammar, NodeVisitor, ParseError
 from parsimonious.nodes import Node
@@ -42,7 +41,7 @@ command_grammar = Grammar(
     equals_value    = ws "=" ws value
     value           = value_in_quotes / number
     value_in_quotes = '"' quoted_value '"'
-    quoted_value    = ~r"[^\"]*"
+    quoted_value    = ~r"([^\"\\]|\\\"|\\\\)*"
     number          = ~r"-?\d+(\.\d+)?"
     ws              = ~r"[\t ]*"
     comment         = ~r"//.*"
@@ -76,7 +75,7 @@ class CommandVisitor(NodeVisitor):
         self.key = letter.text
 
     def visit_quoted_value(self, node: Node, visited_children: List[Node]) -> None:
-        self.value = node.text
+        self.value = parse_value(node.text)
 
     def visit_number(self, node: Node, visited_children: List[Node]) -> None:
         self.value = node.text
@@ -99,7 +98,9 @@ def parse_command(line: str) -> Tuple[str, Optional[str]]:
         return CommandVisitor().visit(command_grammar.parse(line))  # type: ignore
     except ParseError:
         if line.strip()[0] == "#":
-            raise ParseError(f"Invalid command syntax : {line}") from None
+            raise ParseError(
+                "Line starts with '#' but it couldn't be parsed as a valid command"
+            ) from None
         else:
             raise
 
@@ -113,8 +114,38 @@ def dump_command(key: str, value: Any = None) -> str:
     if isinstance(value, Number):
         value_part = f"={value}"
     elif value is not None:
-        value_part = f'="{value}"'
+        escaped = dump_value(str(value))
+        value_part = f'="{escaped}"'
     else:
         value_part = ""
 
     return key_part + value_part
+
+
+BACKSLASH = "\\"
+
+
+def parse_value(escaped: str) -> str:
+    """Unescapes a backslash-escaped string"""
+    res = []
+    i = 0
+    while i < len(escaped):
+        char = escaped[i]
+        if char == BACKSLASH:
+            if i + 1 == len(escaped):
+                raise ValueError("backslash at end of string")
+            else:
+                i += 1
+
+        res.append(escaped[i])
+        i += 1
+
+    return "".join(res)
+
+
+ESCAPE_TABLE = str.maketrans({'"': BACKSLASH + '"', BACKSLASH: BACKSLASH + BACKSLASH})
+
+
+def dump_value(value: str) -> str:
+    """Escapes backslashes and " from a string"""
+    return value.translate(ESCAPE_TABLE)
