@@ -1,4 +1,5 @@
 """Collection of parsing tools that are common to all the jubeat analyser formats"""
+
 import re
 import warnings
 from collections import Counter
@@ -6,17 +7,7 @@ from copy import deepcopy
 from dataclasses import astuple, dataclass
 from decimal import Decimal
 from itertools import product, zip_longest
-from typing import (
-    AbstractSet,
-    Dict,
-    Iterator,
-    List,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import AbstractSet, Dict, List, Optional, Set, Tuple, Union
 
 import constraint
 from parsimonious import Grammar, NodeVisitor, ParseError
@@ -35,10 +26,10 @@ from .symbols import (
 
 DIFFICULTIES = {1: "BSC", 2: "ADV", 3: "EXT"}
 
-SYMBOL_TO_DECIMAL_TIME = {c: Decimal("0.25") * i for i, c in enumerate(NOTE_SYMBOLS)}
+SYMBOL_TO_BEATS_TIME = {c: BeatsTime("1/4") * i for i, c in enumerate(NOTE_SYMBOLS)}
 
-CIRCLE_FREE_TO_DECIMAL_TIME = {
-    c: Decimal("0.25") * i for i, c in enumerate(CIRCLE_FREE_SYMBOLS)
+CIRCLE_FREE_TO_BEATS_TIME = {
+    c: BeatsTime("1/4") * i for i, c in enumerate(CIRCLE_FREE_SYMBOLS)
 }
 
 CIRCLE_FREE_TO_NOTE_SYMBOL = dict(zip(CIRCLE_FREE_SYMBOLS, NOTE_SYMBOLS))
@@ -74,6 +65,13 @@ EMPTY_BEAT_SYMBOLS = {
 }
 
 
+SEPARATOR = re.compile(r"--.*")
+
+
+def is_separator(line: str) -> bool:
+    return bool(SEPARATOR.match(line))
+
+
 double_column_chart_line_grammar = Grammar(
     r"""
     line            = ws position_part ws (timing_part ws)? comment?
@@ -93,6 +91,32 @@ class DoubleColumnChartLine:
     def __str__(self) -> str:
         return f"{self.position} |{self.timing}|"
 
+    def raise_if_unfit(self, bytes_per_panel: int) -> None:
+        self.raise_if_position_unfit(bytes_per_panel)
+        self.raise_if_timing_unfit(bytes_per_panel)
+
+    def raise_if_position_unfit(self, bytes_per_panel: int) -> None:
+        expected_length = 4 * bytes_per_panel
+        actual_length = len(self.position.encode("shift-jis-2004"))
+        if expected_length != actual_length:
+            raise SyntaxError(
+                f"Invalid position part. Since #bpp={bytes_per_panel}, the "
+                f"position part of a line should be {expected_length} bytes long, "
+                f"but {self.position!r} is {actual_length} bytes long"
+            )
+
+    def raise_if_timing_unfit(self, bytes_per_panel: int) -> None:
+        if self.timing is None:
+            return
+
+        length = len(self.timing.encode("shift-jis-2004"))
+        if length % bytes_per_panel != 0:
+            raise SyntaxError(
+                f"Invalid timing part. Since #bpp={bytes_per_panel}, the timing "
+                f"part of a line should be divisible by {bytes_per_panel}, but "
+                f"{self.timing!r} is {length} bytes long so it's not"
+            )
+
 
 class DoubleColumnChartLineVisitor(NodeVisitor):
     def __init__(self) -> None:
@@ -145,7 +169,10 @@ def split_double_byte_line(line: str) -> List[str]:
     """
     encoded_line = line.encode("shift-jis-2004")
     if len(encoded_line) % 2 != 0:
-        raise ValueError(f"Invalid chart line : {line}")
+        raise ValueError(
+            "Line of odd length encountered while trying to split a double-byte "
+            f"line : {line!r}"
+        )
     symbols = []
     for i in range(0, len(encoded_line), 2):
         symbols.append(encoded_line[i : i + 2].decode("shift-jis-2004"))
@@ -166,7 +193,8 @@ class UnfinishedLongNote:
     def ends_at(self, end: BeatsTime) -> LongNote:
         if end < self.time:
             raise ValueError(
-                f"Invalid end time ({end}) for long note starting at {self.time}"
+                "Invalid end time. A long note starting at "
+                f"{self.time} cannot end at {end} (which is earlier)"
             )
         return LongNote(
             time=self.time,
@@ -215,7 +243,8 @@ Candidates = Dict[NotePosition, Set[NotePosition]]
 
 
 def pick_correct_long_note_candidates(
-    arrow_to_note_candidates: Candidates, bloc: List[List[str]],
+    arrow_to_note_candidates: Candidates,
+    bloc: List[List[str]],
 ) -> Solution:
     """Believe it or not, assigning each arrow to a valid note candidate
     involves whipping out a CSP solver.
@@ -228,7 +257,7 @@ def pick_correct_long_note_candidates(
     solutions: List[Solution] = problem.getSolutions()
     if not solutions:
         raise SyntaxError(
-            "Invalid long note arrow pattern in bloc :\n"
+            "Impossible arrow pattern found in block :\n"
             + "\n".join("".join(line) for line in bloc)
         )
     solution = min(solutions, key=long_note_solution_heuristic)
@@ -263,14 +292,14 @@ def is_simple_solution(solution: Solution, domains: Candidates) -> bool:
 class JubeatAnalyserParser:
     def __init__(self) -> None:
         self.music: Optional[str] = None
-        self.symbols = deepcopy(SYMBOL_TO_DECIMAL_TIME)
-        self.section_starting_beat = Decimal("0")
+        self.symbols = deepcopy(SYMBOL_TO_BEATS_TIME)
+        self.section_starting_beat = BeatsTime(0)
         self.current_tempo = Decimal(120)
         self.timing_events: List[BPMEvent] = []
         self.offset = 0
-        self.beats_per_section = Decimal(4)
+        self.beats_per_section = BeatsTime(4)
         self.bytes_per_panel = 2
-        self.level = 1
+        self.level = Decimal(1)
         self.difficulty: Optional[str] = None
         self.title: Optional[str] = None
         self.artist: Optional[str] = None
@@ -283,7 +312,7 @@ class JubeatAnalyserParser:
         try:
             method = getattr(self, f"do_{command}")
         except AttributeError:
-            raise SyntaxError(f"Unknown analyser command : {command}") from None
+            raise SyntaxError(f"Unknown jubeat analyser command : {command}") from None
 
         if value is not None:
             method(value)
@@ -291,7 +320,7 @@ class JubeatAnalyserParser:
             method()
 
     def do_b(self, value: str) -> None:
-        self.beats_per_section = Decimal(value)
+        self.beats_per_section = decimal_to_beats(Decimal(value))
 
     def do_m(self, value: str) -> None:
         self.music = value
@@ -305,7 +334,10 @@ class JubeatAnalyserParser:
     def do_t(self, value: str) -> None:
         self.current_tempo = Decimal(value)
         self.timing_events.append(
-            BPMEvent(BeatsTime(self.section_starting_beat), BPM=self.current_tempo,)
+            BPMEvent(
+                time=self._current_beat(),
+                BPM=self.current_tempo,
+            )
         )
 
     def do_pw(self, value: str) -> None:
@@ -315,7 +347,7 @@ class JubeatAnalyserParser:
     do_ph = do_pw
 
     def do_lev(self, value: str) -> None:
-        self.level = int(value)
+        self.level = Decimal(value)
 
     def do_dif(self, value: str) -> None:
         dif = int(value)
@@ -341,7 +373,7 @@ class JubeatAnalyserParser:
         if bpp not in (1, 2):
             raise ValueError(f"Unexcpected bpp value : {value}")
         elif self.circle_free and bpp == 1:
-            raise ValueError("#bpp can only be 2 when #circlefree is activated")
+            raise ValueError("Can't set #bpp to 1 when #circlefree is on")
         else:
             self.bytes_per_panel = int(value)
 
@@ -349,18 +381,17 @@ class JubeatAnalyserParser:
         self.hold_by_arrow = int(value) == 1
 
     def do_holdbytilde(self, value: str) -> None:
-        if int(value):
-            raise ValueError("jubeatools does not support #holdbytilde")
+        raise NotImplementedError("jubeatools does not support #holdbytilde")
 
     def do_circlefree(self, raw_value: str) -> None:
         activate = bool(int(raw_value))
         if activate and self.bytes_per_panel != 2:
-            raise ValueError("#circlefree can only be activated when #bpp=2")
+            raise ValueError("#circlefree can only be on when #bpp=2")
         self.circle_free = activate
 
     def _wrong_format(self, f: str) -> None:
         raise ValueError(
-            f"{f} command indicates this file uses another jubeat analyser "
+            f"{f} command means that this file uses another jubeat analyser "
             "format than the one the currently selected parser is designed for"
         )
 
@@ -385,12 +416,12 @@ class JubeatAnalyserParser:
                 f"should be {bpp} bytes long but '{symbol}' is {length_as_shift_jis}"
             )
         if timing > self.beats_per_section:
-            message = (
-                "Invalid symbol definition conscidering the number of beats per section :\n"
-                f"*{symbol}:{timing}"
+            raise ValueError(
+                f"Invalid symbol definition. Since sections only last "
+                f"{self.beats_per_section} beats, a symbol cannot happen "
+                f"afterwards at {timing}"
             )
-            raise ValueError(message)
-        self.symbols[symbol] = timing
+        self.symbols[symbol] = decimal_to_beats(timing)
 
     def is_short_line(self, line: str) -> bool:
         return len(line.encode("shift-jis-2004")) < self.bytes_per_panel * 4
@@ -401,6 +432,17 @@ class JubeatAnalyserParser:
         else:
             return list(line)
 
+    def raise_if_separator(self, line: str, format_: str) -> None:
+        if is_separator(line):
+            raise SyntaxError(
+                'Found a separator line (starting with "--") but the file '
+                f"indicates it's using {format_} format, if the file is actually "
+                f"in mono-column format (1列形式) there should be no {format_} line"
+            )
+
+    def _current_beat(self) -> BeatsTime:
+        raise NotImplementedError
+
 
 @dataclass
 class DoubleColumnFrame:
