@@ -1,19 +1,14 @@
 from collections import ChainMap, defaultdict
-from copy import deepcopy
 from dataclasses import dataclass, field
-from decimal import Decimal
 from fractions import Fraction
-from functools import partial
 from io import StringIO
-from itertools import chain, zip_longest
+from itertools import zip_longest
 from math import ceil
-from pathlib import Path
-from typing import Callable, Dict, Iterator, List, Optional, Set, Tuple, Union, cast
+from typing import Dict, Iterator, List, Union
 
 from more_itertools import chunked, collapse, intersperse, mark_ends, windowed
-from sortedcontainers import SortedKeyList
 
-from jubeatools.formats.filetypes import ChartFile, JubeatFile
+from jubeatools.formats.filetypes import ChartFile
 from jubeatools.song import (
     BeatsTime,
     Chart,
@@ -26,22 +21,17 @@ from jubeatools.song import (
 )
 from jubeatools.version import __version__
 
-from ..command import dump_command
 from ..dump_tools import (
-    BEATS_TIME_TO_SYMBOL,
-    COMMAND_ORDER,
     DEFAULT_EXTRA_SYMBOLS,
     DIRECTION_TO_ARROW,
     DIRECTION_TO_LINE,
     NOTE_TO_CIRCLE_FREE_SYMBOL,
     JubeatAnalyserDumpedSection,
     LongNoteEnd,
-    SortedDefaultDict,
     create_sections_from_chart,
-    fraction_to_decimal,
     jubeat_analyser_file_dumper,
 )
-from ..symbols import CIRCLE_FREE_SYMBOLS, NOTE_SYMBOLS
+from ..symbols import NOTE_SYMBOLS
 
 AnyNote = Union[TapNote, LongNote, LongNoteEnd]
 
@@ -54,20 +44,29 @@ class Frame:
     positions: Dict[NotePosition, str] = field(default_factory=dict)
     bars: Dict[int, Dict[int, str]] = field(default_factory=dict)
 
-    def dump(self, length: Decimal) -> Iterator[str]:
-        # Check that bars are contiguous
-        for a, b in windowed(sorted(self.bars), 2):
-            if b is not None and a is not None and b - a != 1:
-                raise ValueError("Frame has discontinuous bars")
-        # Check all bars are in the same 4-bar group
-        if self.bars.keys() != set(bar % 4 for bar in self.bars):
-            raise ValueError("Frame contains bars from different 4-bar groups")
-
+    def dump(self, length: BeatsTime) -> Iterator[str]:
+        self.raise_if_unfit()
         for pos, bar in zip_longest(self.dump_positions(), self.dump_bars(length)):
             if bar is None:
                 bar = ""
             yield f"{pos} {bar}"
 
+    def raise_if_unfit(self) -> None:
+        if not self.bars_are_contiguous():
+            raise ValueError("Frame has discontinuous bars")
+        if not self.all_bars_are_from_the_same_group():
+            raise ValueError("Frame contains bars from different 4-bar groups")
+
+    def bars_are_contiguous(self) -> bool:
+        return all(
+            b - a == 1
+            for a, b in windowed(sorted(self.bars), 2)
+            if b is not None and a is not None
+        )
+
+    def all_bars_are_from_the_same_group(self) -> bool:
+        return self.bars.keys() == set(bar % 4 for bar in self.bars)
+
     def dump_positions(self) -> Iterator[str]:
         for y in range(4):
             yield "".join(
@@ -75,7 +74,7 @@ class Frame:
                 for x in range(4)
             )
 
-    def dump_bars(self, length: Decimal) -> Iterator[str]:
+    def dump_bars(self, length: BeatsTime) -> Iterator[str]:
         all_bars = []
         for i in range(ceil(length * 4)):
             bar_index = i // 4
@@ -121,9 +120,7 @@ class MemoDumpedSection(JubeatAnalyserDumpedSection):
                     chosen_symbols[time_in_section] = symbol
                     bars[bar_index][time_index] = symbol
             elif time_in_section not in self.symbols:
-                raise ValueError(
-                    f"No symbol defined for time in section : {time_in_section}"
-                )
+                raise ValueError(f"No symbol defined for time : {time_in_section}")
 
         # Create frame by bar
         section_symbols = ChainMap(chosen_symbols, self.symbols)
@@ -212,6 +209,10 @@ def _raise_if_unfit_for_memo(chart: Chart, timing: Timing, circle_free: bool) ->
         )
 
 
+def _section_factory(b: BeatsTime) -> MemoDumpedSection:
+    return MemoDumpedSection(current_beat=b)
+
+
 def _dump_memo_chart(
     difficulty: str,
     chart: Chart,
@@ -250,8 +251,9 @@ def _dump_memo_chart(
     file = StringIO()
     file.write(f"// Converted using jubeatools {__version__}\n")
     file.write(f"// https://github.com/Stepland/jubeatools\n\n")
-    for _, section in sections.items():
-        file.write(section.render(circle_free) + "\n\n")
+    file.write(
+        "\n\n".join(section.render(circle_free) for _, section in sections.items())
+    )
 
     return file
 
@@ -262,7 +264,11 @@ def _dump_memo_internal(song: Song, circle_free: bool) -> List[ChartFile]:
         timing = chart.timing or song.global_timing
         assert timing is not None
         contents = _dump_memo_chart(
-            difficulty, chart, song.metadata, timing, circle_free,
+            difficulty,
+            chart,
+            song.metadata,
+            timing,
+            circle_free,
         )
         files.append(ChartFile(contents, song, difficulty, chart))
 
