@@ -1,19 +1,14 @@
-from collections import ChainMap, defaultdict
-from copy import deepcopy
+from collections import defaultdict
 from dataclasses import dataclass, field
-from decimal import Decimal
 from fractions import Fraction
-from functools import partial
 from io import StringIO
-from itertools import chain, zip_longest
+from itertools import zip_longest
 from math import ceil
-from pathlib import Path
-from typing import Dict, Iterator, List, Optional, Set, Tuple, Union
+from typing import Dict, Iterator, List, Union
 
-from more_itertools import chunked, collapse, intersperse, mark_ends, windowed
-from sortedcontainers import SortedKeyList
+from more_itertools import collapse, intersperse, mark_ends, windowed
 
-from jubeatools.formats.filetypes import ChartFile, JubeatFile
+from jubeatools.formats.filetypes import ChartFile
 from jubeatools.song import (
     BeatsTime,
     Chart,
@@ -27,22 +22,16 @@ from jubeatools.song import (
 from jubeatools.utils import lcm
 from jubeatools.version import __version__
 
-from ..command import dump_command
 from ..dump_tools import (
-    BEATS_TIME_TO_SYMBOL,
-    COMMAND_ORDER,
-    DEFAULT_EXTRA_SYMBOLS,
     DIRECTION_TO_ARROW,
     DIRECTION_TO_LINE,
     NOTE_TO_CIRCLE_FREE_SYMBOL,
     JubeatAnalyserDumpedSection,
     LongNoteEnd,
-    SortedDefaultDict,
     create_sections_from_chart,
-    fraction_to_decimal,
     jubeat_analyser_file_dumper,
 )
-from ..symbols import CIRCLE_FREE_SYMBOLS, NOTE_SYMBOLS
+from ..symbols import NOTE_SYMBOLS
 
 AnyNote = Union[TapNote, LongNote, LongNoteEnd]
 
@@ -55,7 +44,7 @@ class Frame:
     positions: Dict[NotePosition, str] = field(default_factory=dict)
     bars: Dict[int, List[str]] = field(default_factory=dict)
 
-    def dump(self, length: Decimal) -> Iterator[str]:
+    def dump(self, length: BeatsTime) -> Iterator[str]:
         # Check that bars are contiguous
         for a, b in windowed(sorted(self.bars), 2):
             if a is not None and b is not None:
@@ -77,7 +66,7 @@ class Frame:
                 for x in range(4)
             )
 
-    def dump_bars(self, length: Decimal) -> Iterator[str]:
+    def dump_bars(self, length: BeatsTime) -> Iterator[str]:
         for i in range(ceil(length)):
             if i in self.bars:
                 yield f"|{''.join(self.bars[i])}|"
@@ -110,7 +99,9 @@ class Memo1DumpedSection(JubeatAnalyserDumpedSection):
         symbols_iterator = iter(NOTE_SYMBOLS)
         for bar_index in range(ceil(self.length)):
             notes = notes_by_bar.get(bar_index, [])
-            bar_length = lcm(*(note.time.denominator for note in notes))
+            bar_length = lcm(
+                *((note.time - self.current_beat).denominator for note in notes)
+            )
             if bar_length < 3:
                 bar_length = 4
             bar_dict: Dict[int, str] = {}
@@ -215,6 +206,10 @@ def _raise_if_unfit_for_memo1(
         )
 
 
+def _section_factory(b: BeatsTime) -> Memo1DumpedSection:
+    return Memo1DumpedSection(current_beat=b)
+
+
 def _dump_memo1_chart(
     difficulty: str,
     chart: Chart,
@@ -236,8 +231,9 @@ def _dump_memo1_chart(
     file = StringIO()
     file.write(f"// Converted using jubeatools {__version__}\n")
     file.write(f"// https://github.com/Stepland/jubeatools\n\n")
-    for _, section in sections.items():
-        file.write(section.render(circle_free) + "\n")
+    file.write(
+        "\n\n".join(section.render(circle_free) for _, section in sections.items())
+    )
 
     return file
 
@@ -248,7 +244,11 @@ def _dump_memo1_internal(song: Song, circle_free: bool) -> List[ChartFile]:
         timing = chart.timing or song.global_timing
         assert timing is not None
         contents = _dump_memo1_chart(
-            difficulty, chart, song.metadata, timing, circle_free,
+            difficulty,
+            chart,
+            song.metadata,
+            timing,
+            circle_free,
         )
         files.append(ChartFile(contents, song, difficulty, chart))
 
