@@ -1,16 +1,12 @@
-import warnings
-from collections import ChainMap
 from copy import deepcopy
 from dataclasses import astuple, dataclass
 from decimal import Decimal
 from functools import reduce
-from itertools import chain, product, zip_longest
+from itertools import product
 from pathlib import Path
-from typing import Dict, Iterator, List, Mapping, Optional, Set, Tuple, Union
+from typing import Dict, Iterator, List, Mapping, Set, Tuple, Union
 
-import constraint
-from more_itertools import collapse, mark_ends
-from parsimonious import Grammar, NodeVisitor, ParseError
+from more_itertools import mark_ends
 
 from jubeatools.song import (
     BeatsTime,
@@ -29,39 +25,33 @@ from jubeatools.utils import none_or
 from ..command import is_command, parse_command
 from ..files import load_files
 from ..load_tools import (
-    CIRCLE_FREE_TO_DECIMAL_TIME,
     CIRCLE_FREE_TO_NOTE_SYMBOL,
     EMPTY_BEAT_SYMBOLS,
-    LONG_ARROWS,
-    LONG_DIRECTION,
     DoubleColumnChartLine,
     DoubleColumnFrame,
     JubeatAnalyserParser,
     UnfinishedLongNote,
-    decimal_to_beats,
     find_long_note_candidates,
     is_double_column_chart_line,
     is_empty_line,
-    is_simple_solution,
-    long_note_solution_heuristic,
     parse_double_column_chart_line,
     pick_correct_long_note_candidates,
-    split_double_byte_line,
 )
-from ..symbol_definition import is_symbol_definition, parse_symbol_definition
-from ..symbols import CIRCLE_FREE_SYMBOLS, NOTE_SYMBOLS
+from ..symbols import CIRCLE_FREE_SYMBOLS
 
 
 class Memo1Frame(DoubleColumnFrame):
     @property
-    def duration(self) -> Decimal:
-        return Decimal(len(self.timing_part))
+    def duration(self) -> BeatsTime:
+        # This is wrong for the last frame in a section if the section has a
+        # length in beats that's not an integer
+        return BeatsTime(len(self.timing_part))
 
 
 @dataclass
 class Memo1LoadedSection:
     frames: List[Memo1Frame]
-    length: Decimal
+    length: BeatsTime
     tempo: Decimal
 
     def __str__(self) -> str:
@@ -76,35 +66,56 @@ class Memo1LoadedSection:
 
 
 class Memo1Parser(JubeatAnalyserParser):
+
+    FORMAT_TAG = "#memo1"
+
     def __init__(self) -> None:
         super().__init__()
         self.current_chart_lines: List[DoubleColumnChartLine] = []
-        self.frames: List[Memo1Frame] = []
+        self.current_frames: List[Memo1Frame] = []
         self.sections: List[Memo1LoadedSection] = []
 
     def do_memo1(self) -> None:
         ...
 
+    def do_b(self, value: str) -> None:
+        """Because of the way the parser works,
+        b= commands must mark the end of a section to be properly taken into
+        account"""
+        if self.current_chart_lines:
+            raise SyntaxError("Found a b= command before the end of a frame")
+        if self.current_frames and self._frames_duration() < self.beats_per_section:
+            raise SyntaxError("Found a b= command before the end of a section")
+
+        self._push_section()
+        super().do_b(value)
+
     def do_bpp(self, value: str) -> None:
-        if self.sections or self.frames:
+        if self.sections or self.current_frames:
             raise ValueError("jubeatools does not handle changes of #bpp halfway")
         else:
             self._do_bpp(value)
 
     def append_chart_line(self, line: DoubleColumnChartLine) -> None:
-        if len(line.position.encode("shift-jis-2004")) != 4 * self.bytes_per_panel:
-            raise SyntaxError(
-                f"Invalid chart line for #bpp={self.bytes_per_panel} : {line}"
-            )
-        if line.timing is not None and self.bytes_per_panel == 2:
-            if len(line.timing.encode("shift-jis-2004")) % 2 != 0:
-                raise SyntaxError(f"Invalid chart line for #bpp=2 : {line}")
+        line.raise_if_unfit(self.bytes_per_panel)
         self.current_chart_lines.append(line)
         if len(self.current_chart_lines) == 4:
             self._push_frame()
 
-    def _frames_duration(self) -> Decimal:
-        return sum((frame.duration for frame in self.frames), start=Decimal(0))
+    def _frames_duration(self) -> BeatsTime:
+        return sum(
+            (frame.duration for frame in self.current_frames), start=BeatsTime(0)
+        )
+
+    def _current_beat(self) -> BeatsTime:
+        # If we've already seen enough beats, we need to circumvent the wrong
+        # duration computation
+        if self._frames_duration() >= self.beats_per_section:
+            frames_duration = self.beats_per_section
+        else:
+            frames_duration = self._frames_duration()
+
+        return self.section_starting_beat + frames_duration
 
     def _push_frame(self) -> None:
         position_part = [
@@ -124,18 +135,24 @@ class Memo1Parser(JubeatAnalyserParser):
                 # then the current frame starts a new section
                 self._push_section()
 
-        self.frames.append(frame)
+        self.current_frames.append(frame)
         self.current_chart_lines = []
 
     def _push_section(self) -> None:
+        """Take all currently stacked frames and push them to a new section,
+        Move time forward by the number of beats per section"""
+        if not self.current_frames:
+            raise RuntimeError(
+                "Tried pushing a new section but no frames are currently stacked"
+            )
         self.sections.append(
             Memo1LoadedSection(
-                frames=deepcopy(self.frames),
+                frames=deepcopy(self.current_frames),
                 length=self.beats_per_section,
                 tempo=self.current_tempo,
             )
         )
-        self.frames = []
+        self.current_frames = []
         self.section_starting_beat += self.beats_per_section
 
     def finish_last_few_notes(self) -> None:
@@ -152,6 +169,7 @@ class Memo1Parser(JubeatAnalyserParser):
 
     def load_line(self, raw_line: str) -> None:
         line = raw_line.strip()
+        self.raise_if_separator(line, self.FORMAT_TAG)
         if is_command(line):
             command, value = parse_command(line)
             self.handle_command(command, value)
@@ -172,23 +190,23 @@ class Memo1Parser(JubeatAnalyserParser):
     def _iter_frames(
         self,
     ) -> Iterator[
-        Tuple[Mapping[str, BeatsTime], Memo1Frame, Decimal, Memo1LoadedSection]
+        Tuple[Mapping[str, BeatsTime], Memo1Frame, BeatsTime, Memo1LoadedSection]
     ]:
         """iterate over tuples of
         currently_defined_symbols, frame, section_starting_beat, section"""
         local_symbols = {}
-        section_starting_beat = Decimal(0)
+        section_starting_beat = BeatsTime(0)
         for section in self.sections:
-            frame_starting_beat = Decimal(0)
+            frame_starting_beat = BeatsTime(0)
             for i, frame in enumerate(section.frames):
                 if frame.timing_part:
                     frame_starting_beat = sum(
-                        (f.duration for f in section.frames[:i]), start=Decimal(0)
+                        (f.duration for f in section.frames[:i]), start=BeatsTime(0)
                     )
                     local_symbols = {
                         symbol: BeatsTime(symbol_index, len(bar))
                         + bar_index
-                        + decimal_to_beats(frame_starting_beat)
+                        + frame_starting_beat
                         for bar_index, bar in enumerate(frame.timing_part)
                         for symbol_index, symbol in enumerate(bar)
                         if symbol not in EMPTY_BEAT_SYMBOLS
@@ -228,7 +246,7 @@ class Memo1Parser(JubeatAnalyserParser):
                         continue
 
                 should_skip.add(pos)
-                note_time = decimal_to_beats(section_starting_beat) + symbol_time
+                note_time = section_starting_beat + symbol_time
                 yield unfinished_long.ends_at(note_time)
 
             unfinished_longs = {
@@ -241,14 +259,15 @@ class Memo1Parser(JubeatAnalyserParser):
             )
             if arrow_to_note_candidates:
                 solution = pick_correct_long_note_candidates(
-                    arrow_to_note_candidates, frame.position_part,
+                    arrow_to_note_candidates,
+                    frame.position_part,
                 )
                 for arrow_pos, note_pos in solution.items():
                     should_skip.add(arrow_pos)
                     should_skip.add(note_pos)
                     symbol = frame.position_part[note_pos.y][note_pos.x]
                     symbol_time = currently_defined_symbols[symbol]
-                    note_time = decimal_to_beats(section_starting_beat) + symbol_time
+                    note_time = section_starting_beat + symbol_time
                     unfinished_longs[note_pos] = UnfinishedLongNote(
                         time=note_time, position=note_pos, tail_tip=arrow_pos
                     )
@@ -263,7 +282,7 @@ class Memo1Parser(JubeatAnalyserParser):
                     symbol_time = currently_defined_symbols[symbol]
                 except KeyError:
                     continue
-                note_time = decimal_to_beats(section_starting_beat) + symbol_time
+                note_time = section_starting_beat + symbol_time
                 yield TapNote(note_time, position)
 
     def _iter_notes_without_longs(self) -> Iterator[TapNote]:
@@ -280,20 +299,18 @@ class Memo1Parser(JubeatAnalyserParser):
                     symbol_time = currently_defined_symbols[symbol]
                 except KeyError:
                     continue
-                note_time = decimal_to_beats(section_starting_beat) + symbol_time
+                note_time = section_starting_beat + symbol_time
                 position = NotePosition(x, y)
                 yield TapNote(note_time, position)
 
 
 def _load_memo1_file(lines: List[str]) -> Song:
     parser = Memo1Parser()
-    for i, raw_line in enumerate(lines):
+    for i, raw_line in enumerate(lines, start=1):
         try:
             parser.load_line(raw_line)
         except Exception as e:
-            raise SyntaxError(
-                f"Error while parsing memo line {i} :\n" f"{type(e).__name__}: {e}"
-            ) from None
+            raise SyntaxError(f"On line {i}\n{e}")
 
     parser.finish_last_few_notes()
     metadata = Metadata(
