@@ -1,15 +1,10 @@
-import warnings
-from collections import ChainMap
-from copy import deepcopy
 from dataclasses import astuple, dataclass
 from decimal import Decimal
 from functools import reduce
-from itertools import chain, product, zip_longest
+from itertools import product, zip_longest
 from pathlib import Path
 from typing import Dict, Iterator, List, Mapping, Optional, Set, Tuple, Union
 
-import constraint
-from more_itertools import collapse, mark_ends
 from parsimonious import Grammar, NodeVisitor, ParseError
 from parsimonious.nodes import Node
 
@@ -31,25 +26,15 @@ from jubeatools.utils import none_or
 from ..command import is_command, parse_command
 from ..files import load_files
 from ..load_tools import (
-    CIRCLE_FREE_TO_DECIMAL_TIME,
     CIRCLE_FREE_TO_NOTE_SYMBOL,
     EMPTY_BEAT_SYMBOLS,
-    LONG_ARROWS,
-    LONG_DIRECTION,
     JubeatAnalyserParser,
     UnfinishedLongNote,
-    decimal_to_beats,
     find_long_note_candidates,
-    is_double_column_chart_line,
     is_empty_line,
-    is_simple_solution,
-    long_note_solution_heuristic,
-    parse_double_column_chart_line,
     pick_correct_long_note_candidates,
-    split_double_byte_line,
 )
-from ..symbol_definition import is_symbol_definition, parse_symbol_definition
-from ..symbols import CIRCLE_FREE_SYMBOLS, NOTE_SYMBOLS
+from ..symbols import CIRCLE_FREE_SYMBOLS
 
 
 @dataclass
@@ -98,6 +83,13 @@ class Memo2ChartLine:
     position: str
     timing: Optional[List[str]]
 
+    @property
+    def duration(self) -> BeatsTime:
+        if self.timing:
+            return BeatsTime(1)
+        else:
+            return BeatsTime(0)
+
 
 memo2_chart_line_grammar = Grammar(
     r"""
@@ -179,34 +171,36 @@ class Memo2Frame:
 
 
 class Memo2Parser(JubeatAnalyserParser):
+
+    FORMAT_TAG = "#memo2"
+
     def __init__(self) -> None:
         super().__init__()
         self.current_chart_lines: List[Memo2ChartLine] = []
-        self.current_beat = BeatsTime(0)
         self.frames: List[Memo2Frame] = []
 
     def do_b(self, value: str) -> None:
-        raise ValueError(
+        raise RuntimeError(
             "beat command (b=...) found, this commands cannot be used in #memo2 files"
         )
 
     def do_t(self, value: str) -> None:
         if self.frames:
-            raise ValueError(
+            raise RuntimeError(
                 "tempo command (t=...) found outside of the file header, "
                 "this should not happen in #memo2 files"
             )
         else:
-            self.timing_events.append(BPMEvent(self.current_beat, BPM=Decimal(value)))
+            self.timing_events.append(
+                BPMEvent(self._current_beat(), BPM=Decimal(value))
+            )
 
     def do_r(self, value: str) -> None:
         if self.frames:
-            raise ValueError(
+            raise RuntimeError(
                 "offset increase command (r=...)  found outside of the file "
                 "header, this is not supported by jubeatools"
             )
-        elif self.offset is None:
-            super().do_o(value)
         else:
             super().do_r(value)
 
@@ -218,7 +212,7 @@ class Memo2Parser(JubeatAnalyserParser):
 
     def do_bpp(self, value: Union[int, str]) -> None:
         if self.frames:
-            raise ValueError("jubeatools does not handle changes of #bpp halfway")
+            raise RuntimeError("jubeatools does not handle changes of #bpp halfway")
         else:
             self._do_bpp(value)
 
@@ -255,22 +249,18 @@ class Memo2Parser(JubeatAnalyserParser):
                     in_bar_beat += symbol_duration
                 elif isinstance(event, BPM):
                     self.timing_events.append(
-                        BPMEvent(time=self.current_beat + in_bar_beat, BPM=event.value)
+                        BPMEvent(
+                            time=self._current_beat() + in_bar_beat, BPM=event.value
+                        )
                     )
                 elif isinstance(event, Stop):
-                    time = self.current_beat + in_bar_beat
+                    time = self._current_beat() + in_bar_beat
                     if time != 0:
                         raise ValueError(
                             "Chart contains a pause that's not happening at the "
                             "very first beat, these are not supported by jubeatools"
                         )
-                    if self.offset is None:
-                        self.offset = event.duration
-                    else:
-                        # This could happen if several pauses exist at the first
-                        # beat of the chart or if both an in-bar pause and an
-                        # o=... command exist
-                        self.offset += event.duration
+                    self.offset += event.duration
 
             bar_notes = [e for e in bar if isinstance(e, str)]
             line = Memo2ChartLine(raw_line.position, bar_notes)
@@ -279,9 +269,17 @@ class Memo2Parser(JubeatAnalyserParser):
         if len(self.current_chart_lines) == 4:
             self._push_frame()
 
+    def _current_beat(self) -> BeatsTime:
+        return self._frames_duration() + self._lines_duration()
+
     def _frames_duration(self) -> BeatsTime:
         return sum((frame.duration for frame in self.frames), start=BeatsTime(0))
 
+    def _lines_duration(self) -> BeatsTime:
+        return sum(
+            (line.duration for line in self.current_chart_lines), start=BeatsTime(0)
+        )
+
     def _push_frame(self) -> None:
         position_part = [
             self._split_chart_line(memo_line.position)
@@ -310,6 +308,7 @@ class Memo2Parser(JubeatAnalyserParser):
 
     def load_line(self, raw_line: str) -> None:
         line = raw_line.strip()
+        self.raise_if_separator(line, self.FORMAT_TAG)
         if is_command(line):
             command, value = parse_command(line)
             self.handle_command(command, value)
@@ -327,7 +326,9 @@ class Memo2Parser(JubeatAnalyserParser):
         else:
             yield from self._iter_notes_without_longs()
 
-    def _iter_frames(self,) -> Iterator[Tuple[Mapping[str, BeatsTime], Memo2Frame]]:
+    def _iter_frames(
+        self,
+    ) -> Iterator[Tuple[Mapping[str, BeatsTime], Memo2Frame]]:
         """iterate over tuples of (currently_defined_symbols, frame)"""
         local_symbols = {}
         frame_starting_beat = BeatsTime(0)
@@ -384,7 +385,8 @@ class Memo2Parser(JubeatAnalyserParser):
             )
             if arrow_to_note_candidates:
                 solution = pick_correct_long_note_candidates(
-                    arrow_to_note_candidates, frame.position_part,
+                    arrow_to_note_candidates,
+                    frame.position_part,
                 )
                 for arrow_pos, note_pos in solution.items():
                     should_skip.add(arrow_pos)
@@ -422,13 +424,11 @@ class Memo2Parser(JubeatAnalyserParser):
 
 def _load_memo2_file(lines: List[str]) -> Song:
     parser = Memo2Parser()
-    for i, raw_line in enumerate(lines):
+    for i, raw_line in enumerate(lines, start=1):
         try:
             parser.load_line(raw_line)
         except Exception as e:
-            raise SyntaxError(
-                f"Error while parsing memo line {i} :\n" f"{type(e).__name__}: {e}"
-            ) from None
+            raise SyntaxError(f"On line {i}\n{e}")
 
     parser.finish_last_few_notes()
     metadata = Metadata(
