@@ -1,18 +1,10 @@
-from collections import ChainMap, defaultdict
 from copy import deepcopy
-from dataclasses import dataclass, field
-from decimal import Decimal
-from fractions import Fraction
-from functools import partial
 from io import StringIO
-from itertools import chain
-from pathlib import Path
-from typing import Dict, Iterator, List, Mapping, Optional, Tuple
+from typing import Dict, Iterator, List
 
-from more_itertools import collapse, intersperse, mark_ends, windowed
-from sortedcontainers import SortedKeyList
+from more_itertools import collapse, intersperse, mark_ends
 
-from jubeatools.formats.filetypes import ChartFile, JubeatFile
+from jubeatools.formats.filetypes import ChartFile
 from jubeatools.song import (
     BeatsTime,
     Chart,
@@ -27,18 +19,15 @@ from jubeatools.version import __version__
 
 from ..dump_tools import (
     BEATS_TIME_TO_SYMBOL,
-    COMMAND_ORDER,
     DEFAULT_EXTRA_SYMBOLS,
     DIRECTION_TO_ARROW,
     DIRECTION_TO_LINE,
+    NOTE_TO_CIRCLE_FREE_SYMBOL,
     JubeatAnalyserDumpedSection,
     LongNoteEnd,
-    SortedDefaultDict,
     create_sections_from_chart,
-    fraction_to_decimal,
     jubeat_analyser_file_dumper,
 )
-from ..symbols import CIRCLE_FREE_SYMBOLS, NOTE_SYMBOLS
 
 
 class MonoColumnDumpedSection(JubeatAnalyserDumpedSection):
@@ -55,23 +44,25 @@ class MonoColumnDumpedSection(JubeatAnalyserDumpedSection):
             blocs.append(notes)
         return "\n".join(collapse([intersperse("", blocs), "--"]))
 
-    def _dump_notes(self, circle_free: bool = False,) -> Iterator[str]:
+    def _dump_notes(
+        self,
+        circle_free: bool = False,
+    ) -> Iterator[str]:
         frames: List[Dict[NotePosition, str]] = []
         frame: Dict[NotePosition, str] = {}
         for note in self.notes:
+            time_in_section = note.time - self.current_beat
+            symbol = self.symbols[time_in_section]
             if isinstance(note, LongNote):
                 needed_positions = set(note.positions_covered())
                 if needed_positions & frame.keys():
                     frames.append(frame)
                     frame = {}
-
                 direction = note.tail_direction()
                 arrow = DIRECTION_TO_ARROW[direction]
                 line = DIRECTION_TO_LINE[direction]
                 for is_first, is_last, pos in mark_ends(note.positions_covered()):
                     if is_first:
-                        time_in_section = note.time - self.current_beat
-                        symbol = self.symbols[time_in_section]
                         frame[pos] = symbol
                     elif is_last:
                         frame[pos] = arrow
@@ -81,18 +72,13 @@ class MonoColumnDumpedSection(JubeatAnalyserDumpedSection):
                 if note.position in frame:
                     frames.append(frame)
                     frame = {}
-                time_in_section = note.time - self.current_beat
-                symbol = self.symbols[time_in_section]
                 frame[note.position] = symbol
             elif isinstance(note, LongNoteEnd):
                 if note.position in frame:
                     frames.append(frame)
                     frame = {}
-                time_in_section = note.time - self.current_beat
-                if circle_free:
-                    symbol = CIRCLE_FREE_SYMBOLS[int(time_in_section)]
-                else:
-                    symbol = self.symbols[time_in_section]
+                if circle_free and symbol in NOTE_TO_CIRCLE_FREE_SYMBOL:
+                    symbol = NOTE_TO_CIRCLE_FREE_SYMBOL[symbol]
                 frame[note.position] = symbol
 
         frames.append(frame)
@@ -105,9 +91,7 @@ class MonoColumnDumpedSection(JubeatAnalyserDumpedSection):
             yield "".join(frame.get(NotePosition(x, y), "□") for x in range(4))
 
 
-def _raise_if_unfit_for_mono_column(
-    chart: Chart, timing: Timing, circle_free: bool = False
-) -> None:
+def _raise_if_unfit_for_mono_column(chart: Chart, timing: Timing) -> None:
     if len(timing.events) < 1:
         raise ValueError("No BPM found in file") from None
 
@@ -125,15 +109,9 @@ def _raise_if_unfit_for_mono_column(
             " mono_column format is not supported by jubeatools"
         )
 
-    if circle_free and any(
-        (note.time + note.duration) % BeatsTime(1, 4) != 0
-        for note in chart.notes
-        if isinstance(note, LongNote)
-    ):
-        raise ValueError(
-            "Chart contains long notes whose ending timing aren't"
-            " representable in #circlefree mode"
-        )
+
+def _section_factory(b: BeatsTime) -> MonoColumnDumpedSection:
+    return MonoColumnDumpedSection(current_beat=b)
 
 
 def _dump_mono_column_chart(
@@ -144,10 +122,10 @@ def _dump_mono_column_chart(
     circle_free: bool = False,
 ) -> StringIO:
 
-    _raise_if_unfit_for_mono_column(chart, timing, circle_free)
+    _raise_if_unfit_for_mono_column(chart, timing)
 
     sections = create_sections_from_chart(
-        MonoColumnDumpedSection, chart, difficulty, timing, metadata, circle_free
+        _section_factory, chart, difficulty, timing, metadata, circle_free
     )
 
     # Define extra symbols
@@ -167,7 +145,7 @@ def _dump_mono_column_chart(
     # Actual output to file
     file = StringIO()
     file.write(f"// Converted using jubeatools {__version__}\n")
-    file.write(f"// https://github.com/Stepland/jubeatools\n\n")
+    file.write("// https://github.com/Stepland/jubeatools\n\n")
     for _, section in sections.items():
         file.write(section.render(circle_free) + "\n")
 
@@ -180,7 +158,11 @@ def _dump_mono_column_internal(song: Song, circle_free: bool) -> List[ChartFile]
         timing = chart.timing or song.global_timing
         assert timing is not None
         contents = _dump_mono_column_chart(
-            difficulty, chart, song.metadata, timing, circle_free,
+            difficulty,
+            chart,
+            song.metadata,
+            timing,
+            circle_free,
         )
         files.append(ChartFile(contents, song, difficulty, chart))
 
