@@ -1,10 +1,12 @@
 """
 Hypothesis strategies to generate notes and charts
 """
+
 from decimal import Decimal
-from enum import Enum, Flag, auto
+from enum import Flag, auto
 from itertools import product
-from typing import Any, Callable, List, Optional, Set, TypeVar, Union
+from pathlib import Path
+from typing import Any, Dict, Optional, Set, Union
 
 import hypothesis.strategies as st
 from multidict import MultiDict
@@ -118,7 +120,9 @@ def notes(draw: DrawFunc, options: NoteOption) -> Set[Union[TapNote, LongNote]]:
     if NoteOption.COLLISIONS in options:
         return raw_notes
     else:
-        last_notes = {NotePosition(x, y): None for y, x in product(range(4), range(4))}
+        last_notes: Dict[NotePosition, Optional[BeatsTime]] = {
+            NotePosition(x, y): None for y, x in product(range(4), range(4))
+        }
         notes: Set[Union[TapNote, LongNote]] = set()
         for note in sorted(raw_notes, key=lambda n: (n.time, n.position)):
             last_note_time = last_notes[note.position]
@@ -158,7 +162,10 @@ def bpm_change(draw: DrawFunc) -> BPMEvent:
 
 
 @st.composite
-def timing_info(draw: DrawFunc, bpm_changes: bool = True,) -> Timing:
+def timing_info(
+    draw: DrawFunc,
+    bpm_changes: bool = True,
+) -> Timing:
     first_bpm = draw(bpm_strat())
     first_event = BPMEvent(BeatsTime(0), first_bpm)
     events = [first_event]
@@ -179,7 +186,14 @@ def get_bpm_change_time(b: BPMEvent) -> BeatsTime:
 
 @st.composite
 def chart(draw: DrawFunc, timing_strat: Any, notes_strat: Any) -> Chart:
-    level = draw(st.integers(min_value=0))
+    level = Decimal(
+        draw(
+            st.one_of(
+                st.integers(min_value=0),
+                st.decimals(min_value=0, max_value=10.9, places=1),
+            )
+        )
+    )
     timing = draw(timing_strat)
     notes = draw(notes_strat)
     return Chart(
@@ -201,13 +215,18 @@ def preview(draw: DrawFunc) -> Preview:
 
 
 @st.composite
-def metadata(draw: DrawFunc) -> Metadata:
+def metadata(
+    draw: DrawFunc,
+    text_strat: st.SearchStrategy[str] = st.text(),
+    path_start: st.SearchStrategy[str] = st.text(),
+) -> Metadata:
     return Metadata(
-        title=draw(st.text()),
-        artist=draw(st.text()),
-        audio=draw(st.text()),
-        cover=draw(st.text()),
+        title=draw(text_strat),
+        artist=draw(text_strat),
+        audio=Path(draw(path_start)),
+        cover=Path(draw(path_start)),
         preview=draw(st.one_of(st.none(), preview())),
+        preview_file=draw(path_start),
     )
 
 
